<?php
/*
Plugin Name: Cimy User Manager
Plugin URI: http://www.marcocimmino.net/cimy-wordpress-plugins/cimy-user-manager/
Plugin Description: Import and export users from/to CSV files, supports all WordPress profile data also Cimy User Extra Fields plug-in
Version: 0.9.1
Author: Marco Cimmino
Author URI: mailto:cimmino.marco@gmail.com
*/

/*

Cimy User Manager - Import and export users from/to CSV files
Copyright (c) 2007-2010 Marco Cimmino

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


The full copy of the GNU General Public License is available here: http://www.gnu.org/licenses/gpl.txt

*/

// pre 2.6 compatibility or if not defined
if (!defined("WP_CONTENT_URL"))
	define("WP_CONTENT_URL", get_option("siteurl")."/wp_content");
	
if (!defined("WP_CONTENT_DIR"))
	define("WP_CONTENT_DIR", ABSPATH."/wp_content");

$cum_plugin_name = basename(__FILE__);
$cum_plugin_path = plugin_basename(dirname(__FILE__))."/";
$cum_plugin_dir = WP_CONTENT_DIR."/plugins/".$cum_plugin_path;

$cimy_um_domain = 'cimy_um';
$cimy_um_i18n_is_setup = false;
cimy_um_i18n_setup();

// function that add the submenu under 'Users'
add_action('admin_menu', 'cimy_um_admin_menu_custom');

$userid_code = '% USERID %';
$useremail_code = '% EMAIL %';
$username_code = '% USERNAME %';
$firstname_code = '% FIRSTNAME %';
$lastname_code = '% LASTNAME %';
$nickname_code = '% NICKNAME %';
$website_code = '% WEBSITE %';
$aim_code = '% AIM %';
$yahoo_code = '% YAHOO %';
$jabber_code = '% JABBER %';
$password_code = '% PASSWORD %';
$role_code = "% ROLE %";
$desc_code = "% DESCRIPTION %";

add_action('init', 'cimy_um_download_database');

function cimy_um_download_database() {
	if (isset($_POST["cimy_um_filename"])) {
		if(strpos($_SERVER['HTTP_REFERER'], admin_url('users.php?page=cimy_user_manager')) !== false) {
			$cimy_um_filename = $_POST["cimy_um_filename"];

			header("Pragma: "); // Leave blank for issues with IE
			header("Expires: 0");
			header('Vary: User-Agent');
			header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
			header("Content-Type: text/csv");
			header("Content-Type: application/force-download");
			header("Content-Type: application/download");
			header("Content-Disposition: attachment; filename=\"".basename($cimy_um_filename)."\";");
			header("Content-Transfer-Encoding: binary");
			header("Content-Length: ".filesize($cimy_um_filename));
			readfile($cimy_um_filename);
			exit();
		}
	}
}

function cimy_um_i18n_setup() {
	global $cimy_um_domain, $cimy_um_i18n_is_setup, $cum_plugin_path;

	if ($cimy_um_i18n_is_setup)
		return;

	load_plugin_textdomain($cimy_um_domain, PLUGINDIR.'/'.$cum_plugin_path.'langs', $cum_plugin_path.'langs');
	
	$cimy_um_i18n_is_setup = true;
}


function cimy_um_admin_menu_custom() {
	global $cimy_um_domain;
	
	if (!current_user_can('edit_users'))
		return;
	
	add_submenu_page('users.php', __('Cimy User Manager', $cimy_um_domain), __('Cimy User Manager', $cimy_um_domain), 10, "cimy_user_manager", 'cimy_um_import_export_page');
}

function cimy_um_import_export_page() {
	global $cimy_um_domain, $cimy_uef_name;
	
	if (!current_user_can('edit_users'))
		return;

	$results_import = array();
	$results_export = array();
	
	if (isset($_POST['cimy_um_import']))
		$results_import = cimy_um_import_data();
	
	if (isset($_POST['cimy_um_export']))
		$results_export = cimy_um_export_data();
	
	if (isset($_POST['db_add_users']))
		$db_add_users_sel = ' checked="checked"';
	else
		$db_add_users_sel = "";
	
	if (isset($_POST['db_extra_fields']))
		$db_extra_fields_sel = ' checked="checked"';
	else
		$db_extra_fields_sel = "";
	
	if (isset($_POST['db_date_format']))
		$db_date_format = attribute_escape($_POST['db_date_format']);
	else
		$db_date_format = "%d %B %Y @%H:%M";
	
	if (!isset($cimy_uef_name)) {
		$db_extra_fields_sel = ' disabled="disabled"';
		$db_extra_fields_warning = "<br /><strong>".__("You must activate Cimy User Extra Fields and enable get_CimyFieldValue function to export data from that plug-in", $cimy_um_domain)."</strong>";
	}
	else
		$db_extra_fields_warning = "";
	
	if (isset($_POST["db_field_separator"]))
		$field_separator = attribute_escape(stripslashes($_POST["db_field_separator"]));
	else
		$field_separator = attribute_escape(",");

	if (isset($_POST["db_text_separator"]))
		$text_separator = attribute_escape(stripslashes($_POST["db_text_separator"]));
	else
		$text_separator = attribute_escape("\"");
	?>
	
	<div class="wrap" id="options">
	<?php
		if (function_exists(screen_icon))
			screen_icon("users");
	?>
	<h2><?php _e("Import Users", $cimy_um_domain); ?></h2><?php
	
	// print successes/errors if there are some
	if (count($results_import) > 0) {
	?>
		<br /><div class="updated">
	<?php
		if (isset($results_import["error"])) {
			echo "<h3>".__("ERRORS", $cimy_um_domain)." (".count($results_import["error"]).")</h3>";
	
			foreach ($results_import["error"] as $result)
				echo $result."<br />";
		}
		
		if (isset($results_import["added"])) {
			echo "<h3>".__("USERS SUCCESSFULLY ADDED", $cimy_um_domain)." (".count($results_import["added"]).")</h3>";
	
			foreach ($results_import["added"] as $result)
				echo $result."<br />";
		}
		
		if (isset($results_import["modified"])) {
			echo "<h3>".__("USERS SUCCESSFULLY MODIFIED", $cimy_um_domain)." (".count($results_import["modified"]).")</h3>";
	
			foreach ($results_import["modified"] as $result)
				echo $result."<br />";
		}
		?><br /></div><?php
	}
	?>
	
	<p>
	</p>
	
	<script type="text/javascript" language="javascript">
		function changeEnc(form_id) {
			var browser = navigator.appName;
		
			if (browser == "Microsoft Internet Explorer")
				document.cimy_um_import.encoding = "multipart/form-data";
			else
				document.cimy_um_import.enctype = "multipart/form-data";
		}
	</script>
	
	<form name="cimy_um_import" id="cimy_um_import" method="post" enctype="multipart/form-data">
	
	<table class="form-table">
		<tr>
			<th scope="row" width="40%"><?php _e("Select the CSV file", $cimy_um_domain); ?></th>
			<td width="60%">
				<input type="file" name="db_import" onchange="changeEnc('cimy_um_import');" />
			</td>
		</tr>
		<tr>
			<th scope="row"><?php _e("Select field delimiter", $cimy_um_domain); ?></th>
			<td>
				<input type="text" name="db_field_separator" value="<?php echo $field_separator; ?>" />  <?php _e('If your CSV file is like: "field1","field2" then you need to use comma', $cimy_um_domain); ?>
			</td>
		</tr>
		<tr>
			<th scope="row"><?php _e("Select text delimiter", $cimy_um_domain); ?></th>
			<td>
				<input type="text" name="db_text_separator" value="<?php echo $text_separator; ?>" />  <?php _e('If your CSV file is like: "field1","field2" then you need to use double quote', $cimy_um_domain); ?>
			</td>
		</tr>
		<tr>
			<th scope="row"><?php _e("Create users", $cimy_um_domain); ?></th>
			<td>
				<input type="checkbox" name="db_add_users" value="1"<?php echo $db_add_users_sel; ?> />  <?php _e("Select this option to let the plug-in create users.", $cimy_um_domain); ?><br /><?php _e("<strong>REMEMBER</strong>: for new users <strong>do not</strong> enter any USERID, but <strong>specify</strong> an unique USERNAME and EMAIL", $cimy_um_domain); ?>
			</td>
		</tr>
	</table>
	
	<input type="hidden" name="cimy_um_import" value="1" />
	<p class="submit"><input class="button-primary" type="submit" name="Import" value="<?php _e('Import') ?>" /></p>
	</form>

	<br />
	<?php
		if (function_exists(screen_icon))
			screen_icon("users");
	?>
	<h2><?php _e("Export Users", $cimy_um_domain); ?></h2><?php
	
	// print successes/errors if there are some
	if (count($results_export) > 0) {
	?>
		<br /><div class="updated">
	<?php
		if (isset($results_export["tmp_file"])) {
			echo "<h3>".__("FILE GENERATED", $cimy_um_domain)." (".count($results_export["tmp_file"]).")</h3>";
			echo $results_export["tmp_file"];
			echo "<form name=\"cimy_um_download\" id=\"cimy_um_download\" method=\"post\"><input type=\"hidden\" name=\"cimy_um_filename\" value=\"".$results_export["tmp_file"]."\" /><input type=\"submit\" value=\"".__("Download Export File")."\" /></form>";
			echo "<br />";
		}
		
		if (isset($results_export["error"])) {
			echo "<h3>".__("ERRORS", $cimy_um_domain)." (".count($results_export["error"]).")</h3>";
	
			foreach ($results_export["error"] as $result)
				echo $result."<br />";
		}
		
		if (isset($results_export["exported"])) {
			echo "<h3>".__("USERS SUCCESSFULLY EXPORTED", $cimy_um_domain)." (".count($results_export["exported"]).")</h3>";
	
			foreach ($results_export["exported"] as $result)
				echo $result."<br />";
		}
		?><br /></div><?php
	}
	?>
	
	<p>
	</p>
	
	<form name="cimy_um_export" id="cimy_um_export" method="post">
	
	<table class="form-table">
		<tr>
			<th scope="row" width="40%"><?php _e("Select field delimiter", $cimy_um_domain); ?></th>
			<td width="60%">
				<input type="text" name="db_field_separator" value="<?php echo $field_separator; ?>" />  <?php _e('If your CSV file is like: "field1","field2" then you need to use comma', $cimy_um_domain); ?>
			</td>
		</tr>
		<tr>
			<th scope="row"><?php _e("Select text delimiter", $cimy_um_domain); ?></th>
			<td>
				<input type="text" name="db_text_separator" value="<?php echo $text_separator; ?>" />  <?php _e('If your CSV file is like: "field1","field2" then you need to use double quote', $cimy_um_domain); ?>
			</td>
		</tr>
		<tr>
			<th scope="row"><?php _e("Add also Cimy User Extra Fields data", $cimy_um_domain); ?></th>
			<td>
				<input type="checkbox" name="db_extra_fields" value="1"<?php echo $db_extra_fields_sel; ?> />  <?php _e("Select this option to let the plug-in export also data present into Cimy User Extra Fields", $cimy_um_domain); echo $db_extra_fields_warning; ?>
			</td>
		</tr>
		<tr>
			<th><?php _e("Select date format", $cimy_um_domain); ?></th>
			<td>
				<input type="text" name="db_date_format" value="<?php echo $db_date_format; ?>" />  <?php _e("Select the date/time format to represent registration dates (if any)", $cimy_um_domain); ?><br />
				<?php _e("More info about date/time format are on this", $cimy_um_domain); ?> <a href="http://www.php.net/manual/en/function.strftime.php"><?php _e("LINK", $cimy_um_domain); ?></a>
			</td>
		</tr>
	</table>
	
	<input type="hidden" name="cimy_um_export" value="1" />
	<p class="submit"><input class="button-primary" type="submit" name="Export" value="<?php _e('Export') ?>" /></p>
	</form>
	</div>
	<br />
	<?php
}

function cimy_um_import_data() {
	global $wpdb, $wpdb_data_table, $wpdb_fields_table, $cimy_um_domain;
	global $userid_code, $useremail_code, $username_code, $firstname_code, $lastname_code, $nickname_code, $website_code, $aim_code, $yahoo_code, $jabber_code, $password_code, $role_code, $desc_code;
	
	$results = array();
	
	if (!current_user_can('edit_users'))
		return;

	// try to not timeout
	set_time_limit(0);

	// needed for silly Windows files
	ini_set('auto_detect_line_endings', true);

	$file_type = $_FILES["db_import"]['type'];
	$file_tmp_name = $_FILES["db_import"]['tmp_name'];
	$file_error = $_FILES["db_import"]['error'];

	if (!is_readable($file_tmp_name)) {
		$results["error"][] = __("Cannot open the file", $cimy_um_domain);
		return $results;
	}
	else if (($fh = fopen($file_tmp_name, 'r')) === false) {
		$results["error"][] = __("Cannot open the file", $cimy_um_domain);
		return $results;
	}

	$field_separator = stripslashes($_POST["db_field_separator"]);
	$text_separator = stripslashes($_POST["db_text_separator"]);

	$separator = $text_separator.$field_separator.$text_separator;

	// name of the fields in the file imported
	$fields = explode($separator, fgets($fh));

	// position of special things in $all_data array
	$specials = array();
	
	// position of extra_fields data in $all_data array
	$extra_fields = array();
	
	// ID of the fields in the DB
	$db_extra_fields = array();
	
	$missing_cimy_uef_error = false;
	
	$i = 0;
	$next_field = reset($fields);

	while ($next_field != false) {
		$field = $next_field;
		$next_field = next($fields);

		$field = strtoupper(trim($field, "\n\r"));

		if ($i == 0) {
			$field = substr($field, strlen($text_separator));
			$fields[$i] = $field;
		}

		if ($next_field == false) {
			$last_field = $i;
			$field = substr($field, 0, (-1)*strlen($text_separator));
			$fields[$i] = $field;
		}
		
		switch ($field) {
			case $userid_code:
				$specials["ID"] = $i;
				break;
				
			case $useremail_code:
				$specials["email"] = $i;
				break;
			
			case $username_code:
				$specials["username"] = $i;
				break;
				
			case $firstname_code:
				$specials["firstname"] = $i;
				break;
				
			case $lastname_code:
				$specials["lastname"] = $i;
				break;
				
			case $nickname_code:
				$specials["nickname"] = $i;
				break;
				
			case $website_code:
				$specials["website"] = $i;
				break;
				
			case $aim_code:
				$specials["aim"] = $i;
				break;
				
			case $yahoo_code:
				$specials["yahoo"] = $i;
				break;
				
			case $jabber_code:
				$specials["jabber"] = $i;
				break;
				
			case $password_code:
				$specials["password"] = $i;
				break;
				
			case $role_code:
				$specials["role"] = $i;
				break;
				
			case $desc_code:
				$specials["description"] = $i;
				break;
				
			default:
				$extra_fields[strtoupper($field)] = $i;
				break;
		}
		
		$i++;
	}

	// first line already read, so will be immediately increased to 2
	$line = 1;

	// looping through rows
	while (!feof($fh)) {
		$row = fgets($fh);
		$line++;

		$line_tr = " ".sprintf(__("(line %s)", $cimy_um_domain), $line);

		// remove definitly all new lines and carriage returns
		$row = trim($row, "\n\r");
		
		// remove also space, but not definitly from the row
		if (trim($row) == "")
			continue;

		$all_data = explode($separator, $row);

		$all_data[0] = substr($all_data[0], strlen($text_separator));
		$all_data[$last_field] = substr($all_data[$last_field], 0, (-1)*strlen($text_separator));

		$email = "";
		$username = "";
		$passw = "";
		$wp_user = false;
		$wp_new_user = array();
		$wp_userid = false;
		
		if ((isset($specials["ID"])) && (trim($all_data[$specials["ID"]]) != "")) {
			$wp_userid = intval(trim($all_data[$specials["ID"]]));
			$wp_user = new WP_User($wp_userid);
			
			if ($wp_user->ID != 0) {
				$username = $wp_user->user_login;
				$results["modified"][] = "'".attribute_escape($username)."'".$line_tr;
			}
		}
		
		if ((isset($specials["username"])) && ($username == "")) {
			$username = sanitize_user($all_data[$specials["username"]], true);

			if ($username != $all_data[$specials["username"]])
				$results["error"][] = sprintf(__("username '%s' has some invalid characters, used this username instead: '%s'", $cimy_um_domain), attribute_escape($all_data[$specials["username"]]), attribute_escape($username)).$line_tr;

			if (!is_object($wp_user)) {
				$wp_user = new WP_User($username);
				$wp_userid = intval($wp_user->ID);
		
				if ($wp_user->ID != 0)
					$results["modified"][] = "'".attribute_escape($username)."'".$line_tr;
			}
		}

		// check if user doesn't exist, if not insert!
		if (($wp_user->ID != $wp_userid) || ($wp_user->ID == 0)) {
			// just check what was the error and drop the row as we are not allowed to create new users by the admin
			if (!isset($_POST["db_add_users"])) {
				$new_user_error = "";
				
				if (isset($specials["ID"])) {
					if ($wp_userid == 0)
						$new_user_error = __("userid is missing", $cimy_um_domain);
					else
						$new_user_error = sprintf(__("userid '%s' is not present in the DB", $cimy_um_domain), attribute_escape($wp_userid));
				}
				
				if (isset($specials["username"])) {
					if ($new_user_error != "")
						$new_user_error .= " ".__("and", $cimy_um_domain)." ";
				
					if ($username == "")
						$new_user_error .= __("username is missing", $cimy_um_domain);
					else
						$new_user_error .= sprintf(__("the username '%s' is not present in the DB", $cimy_um_domain), attribute_escape($username));
				}
				
				if ($new_user_error != "")
					$results["error"][] = $new_user_error.$line_tr;
				
				// drop as user is not valid and we cannot create it
				continue;
			}
		
			// check for username: if missing cannot add a new user
			if ($username == "") {
				$results["error"][] = __("username missing cannot add an user without it", $cimy_um_domain).$line_tr;
				continue;
			}
			
			// check for username: if already existing cannot add a new user
			if (username_exists($username) != null) {
				$results["error"][] = sprintf(__("username '%s' already present in the DB (line %s)", $cimy_um_domain), attribute_escape($username)).$line_tr;
				continue;
			}
			
			// check for e-mail: if missing or already present cannot add a new user
			if (isset($specials["email"])) {
				$email = sanitize_email($all_data[$specials["email"]]);

				if ($email != $all_data[$specials["email"]])
					$results["error"][] = sprintf(__("e-mail '%s' has some invalid characters, used this address instead: '%s'", $cimy_um_domain), attribute_escape($all_data[$specials["email"]]), attribute_escape($email)).$line_tr;
			
				if (!email_exists($email)) {
					$wp_new_user["user_email"] = $email;
				}
				else {
					$results["error"][] = sprintf(__("e-mail '%s' already present in the DB, dropped this new user: '%s'", $cimy_um_domain), attribute_escape($email), attribute_escape($username)).$line_tr;
					continue;
				}
			}
			else {
				$results["error"][] = sprintf(__("e-mail field empty, dropped this new user: '%s'", $cimy_um_domain), attribute_escape($username)).$line_tr;
				continue;
			}
			
			// check for e-mail: if empty cannot add a new user
			if ($email == "") {
				$results["error"][] = sprintf(__("e-mail field empty, dropped this new user: '%s'", $cimy_um_domain), attribute_escape($username)).$line_tr;
				continue;
			}
			
			$passw = $all_data[$specials["password"]];
			
			if ($passw == "")
				$passw = $username;
			
			$wp_new_user["user_pass"] = $passw;
			$wp_new_user["user_login"] = $username;
			
			// dropping ID as WordPress/MySQL will assign a correct/new one
			//unset($wp_new_user["ID"]);
			//$results["error"][] = $wp_new_user["ID"];
			
			$wp_userid = wp_insert_user($wp_new_user);
			$wp_user = new WP_User($wp_userid);
			
			$results["added"][] = "'".attribute_escape($username)."'".$line_tr;
		}
		
		if (isset($specials["role"])) {
			switch ($all_data[$specials["role"]]) {
				case "subscriber":
				case "administrator":
				case "editor":
				case "author":
				case "contributor":
					$wp_user->set_role($all_data[$specials["role"]]);
			}
		}

		if (isset($specials["firstname"])) {
			$value = $all_data[$specials["firstname"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["first_name"] = $value;
		}
		
		if (isset($specials["lastname"])) {
			$value = $all_data[$specials["lastname"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["last_name"] = $value;
		}
		
		if (isset($specials["nickname"])) {
			$value = $all_data[$specials["nickname"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["nickname"] = $value;
		}

		// $email == "" means is not a new user
		if ((isset($specials["email"])) && ($email == "")) {
			$value = sanitize_email($all_data[$specials["email"]]);

			if ($value != $all_data[$specials["email"]])
				$results["error"][] = sprintf(__("e-mail '%s' has some invalid characters, used this address instead: '%s'", $cimy_um_domain), attribute_escape($all_data[$specials["email"]]), attribute_escape($value)).$line_tr;

			
			if (!email_exists($value)) {
				//$wp_new_user["ID"] = $wp_userid;
				$wp_new_user["user_email"] = $value;
			}
			else {
				$results["error"][] = sprintf(__("e-mail '%s' already present in the DB, dropped this modification", $cimy_um_domain), attribute_escape($value)).$line_tr;
			}
		}
		
		if (isset($specials["website"])) {
			$value = $all_data[$specials["website"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["user_url"] = $value;
		}
		
		if (isset($specials["aim"])) {
			$value = $all_data[$specials["aim"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["aim"] = $value;
		}
		
		if (isset($specials["yahoo"])) {
			$value = $all_data[$specials["yahoo"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["yim"] = $value;
		}
		
		if (isset($specials["jabber"])) {
			$value = $all_data[$specials["jabber"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["jabber"] = $value;
		}
		
		if (isset($specials["description"])) {
			$value = $all_data[$specials["description"]];

			//$wp_new_user["ID"] = $wp_userid;
			$wp_new_user["description"] = $value;
		}

		if (!empty($wp_new_user)) {
			$wp_new_user["ID"] = $wp_userid;
			wp_update_user($wp_new_user);
		}

		// looping through array that contains extra_fields position in CSV rows
		
		// $fields is the first row
		// every $e_field is the column position in the first row
		// $fields[$e_field] are extra fields' names
		// $db_extra_fields stores all DB ids, key of the array is fields' names
		foreach ($extra_fields as $e_field) {
			if (!isset($db_extra_fields[$fields[$e_field]])) {
				$field_name = trim($fields[$e_field]);
				
				$sql = "SELECT ID,TYPE,LABEL FROM $wpdb_fields_table WHERE NAME=\"".$wpdb->escape(strtoupper($field_name))."\"";
				$result = $wpdb->get_results($sql, ARRAY_A);
				
				if (count($result) > 0)
					$db_extra_fields[$fields[$e_field]] = $result;
				else {
					$db_extra_fields[$fields[$e_field]] = -1;
					
					if (!isset($wpdb_fields_table)) {
						if (!$missing_cimy_uef_error)
							$results["error"][] = __("Cimy User Extra Fields is not active, impossible to import any extra fields data", $cimy_um_domain).$line_tr;
						
						$missing_cimy_uef_error = true;
					} else
						$results["error"][] = sprintf(__("'%s' field doesn't exist", $cimy_um_domain), attribute_escape($field_name)).$line_tr;
				}
			}
			
			if ($db_extra_fields[$fields[$e_field]] != -1) {
				foreach ($db_extra_fields[$fields[$e_field]] as $ef_details) {
					$sql = "SELECT ID FROM $wpdb_data_table WHERE FIELD_ID=".$ef_details["ID"]." AND USER_ID=$wp_userid";
				
					unset($present);
					$present = $wpdb->get_var($sql);
		
					$all_data[$e_field] = $wpdb->escape($all_data[$e_field]);
					$value_to_store = trim($all_data[$e_field]);
					
					if ($ef_details["TYPE"] == "radio") {
						if ($ef_details["LABEL"] == $value_to_store)
							$value_to_store = "selected";
						else
							$value_to_store = "";
					}
					
					if ($ef_details["TYPE"] == "checkbox") {
						if ((strtoupper($value_to_store) == "YES") || ($value_to_store == "1"))
							$value_to_store = "YES";
						else
							$value_to_store = "NO";
					}
				
					$value_to_store = "\"".$value_to_store."\"";
		
					if (isset($present))
						$sql = "UPDATE $wpdb_data_table SET VALUE=".$value_to_store." WHERE USER_ID=$wp_userid AND FIELD_ID=".$ef_details["ID"];
					else
						$sql = "INSERT INTO $wpdb_data_table (USER_ID, FIELD_ID, VALUE) VALUES ($wp_userid, ".$ef_details["ID"].", ".$value_to_store.")";
	
					$wpdb->query($sql);
				}
			}
		}
	}
	
	fclose($fh);
	
	return $results;
}

function cimy_um_export_data() {
	global $wpdb, $wpdb_data_table, $wpdb_fields_table, $cimy_um_domain;
	global $userid_code, $useremail_code, $username_code, $firstname_code, $lastname_code, $nickname_code, $website_code, $aim_code, $yahoo_code, $jabber_code, $password_code, $role_code, $desc_code, $cimy_uef_name;
	
	$results = array();
	
	if (!current_user_can('edit_users'))
		return;
	
	set_time_limit(0);

	$field_separator = stripslashes($_POST["db_field_separator"]);
	$text_separator = stripslashes($_POST["db_text_separator"]);
	
	if ((isset($_POST["db_extra_fields"])) && (isset($cimy_uef_name))) {
		global $wpdb_data_table;
		$extra_fields = get_cimyFields();
		$all_radio_fields = array();
	}
	else {
		$extra_fields = false;
	}
	
	if (isset($_POST['db_date_format']))
		$db_date_format = $_POST['db_date_format'];
	else
		$db_date_format = "";
	
	$all_users = get_users_of_blog();
	$tmpdir = cimy_um_get_temp_dir();
	$tmpfile = $tmpdir."/cimy_um_exported_users-".date("Ymd-His").".csv";
	
	$fd_tmp_file = fopen($tmpfile, "w");
	
	$line = $text_separator.$userid_code.$text_separator.
		$field_separator.$text_separator.$username_code.$text_separator.
		$field_separator.$text_separator.$role_code.$text_separator.
		$field_separator.$text_separator.$firstname_code.$text_separator.
		$field_separator.$text_separator.$lastname_code.$text_separator.
		$field_separator.$text_separator.$nickname_code.$text_separator.
		$field_separator.$text_separator.$useremail_code.$text_separator.
		$field_separator.$text_separator.$website_code.$text_separator.
		$field_separator.$text_separator.$aim_code.$text_separator.
		$field_separator.$text_separator.$yahoo_code.$text_separator.
		$field_separator.$text_separator.$jabber_code.$text_separator.
		$field_separator.$text_separator.$desc_code.$text_separator;
	
	if ($extra_fields) {
		foreach ($extra_fields as $field) {
			// avoid radio fields duplicates
			if ($field["TYPE"] == "radio") {
				if (in_array($field["NAME"], $all_radio_fields))
					continue;
				else
					$all_radio_fields[] = $field["NAME"];
			}
			
			$line .= $field_separator.$text_separator.$field["NAME"].$text_separator;
		}
	}
	
	$line .= "\r";
	
	fwrite($fd_tmp_file, $line);
	
	$results["exported"] = array();
	
	foreach ($all_users as $user) {
		$current_user = new WP_User($user->user_id);
		
		$results["exported"][] = $current_user->user_login;

		$line = $text_separator.$current_user->ID.$text_separator.
				$field_separator.$text_separator.$current_user->user_login.$text_separator.
				$field_separator.$text_separator.$current_user->roles[0].$text_separator.
				$field_separator.$text_separator.$current_user->first_name.$text_separator.
				$field_separator.$text_separator.$current_user->last_name.$text_separator.
				$field_separator.$text_separator.$current_user->nickname.$text_separator.
				$field_separator.$text_separator.$current_user->user_email.$text_separator.
				$field_separator.$text_separator.$current_user->user_url.$text_separator.
				$field_separator.$text_separator.$current_user->aim.$text_separator.
				$field_separator.$text_separator.$current_user->yim.$text_separator.
				$field_separator.$text_separator.$current_user->jabber.$text_separator.
				$field_separator.$text_separator.$current_user->user_description.$text_separator;
		
		if ($extra_fields) {
			$all_radio_fields = array();
			
			foreach ($extra_fields as $field) {
				// avoid radio fields duplicates
				if ($field["TYPE"] == "radio") {
					if (in_array($field["NAME"], $all_radio_fields))
						continue;
					else
						$all_radio_fields[] = $field["NAME"];
				}
				
				$ef_db = get_cimyFieldValue($current_user->ID, $field["NAME"]);
				
				if ($field["TYPE"] == "registration-date")
					$ef_db = cimy_get_formatted_date($ef_db, $db_date_format);
				
				$line .= $field_separator.$text_separator.$ef_db.$text_separator;
			}
		}
	
		$line .= "\r";
		
		fwrite($fd_tmp_file, $line);
	}
	
	fclose($fd_tmp_file);

	$results["tmp_file"] = $tmpfile;

	return $results;
}

function cimy_um_get_temp_dir() {
        // Try to get from environment variable
	if ( !empty($_ENV['TMP']) )
	{
		return realpath( $_ENV['TMP'] );
	}
	else if ( !empty($_ENV['TMPDIR']) )
	{
		return realpath( $_ENV['TMPDIR'] );
	}
	else if ( !empty($_ENV['TEMP']) )
	{
		return realpath( $_ENV['TEMP'] );
	}

        // Detect by creating a temporary file
	else
	{
		// Try to use system's temporary directory
		// as random name shouldn't exist
		$temp_file = tempnam( md5(uniqid(rand(), TRUE)), '' );
		if ( $temp_file )
		{
			$temp_dir = realpath( dirname($temp_file) );
			unlink( $temp_file );
			return $temp_dir;
		}
		else
		{
			return FALSE;
		}
	}
}

?>
